@@ -28,7 +28,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD $iptcDigestInfo);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.65';
+$VERSION = '1.66';
 
 sub ProcessPhotoshop($$$);
 sub WritePhotoshop($$$);
@@ -248,11 +248,12 @@ my %unicodeString = (
         Protected => 1,
         Notes => q{
             this tag indicates provides a way for XMP-aware applications to indicate
-            that the XMP is synchronized with the IPTC.  When writing, special values of
-            "new" and "old" represent the digests of the IPTC from the edited and
-            original files respectively, and are undefined if the IPTC does not exist in
-            the respective file.  Set this to "new" as an indication that the XMP is
-            synchronized with the IPTC
+            that the XMP is synchronized with the IPTC.  The MWG recommendation is to
+            ignore the XMP if IPTCDigest exists and doesn't match the CurrentIPTCDigest.
+            When writing, special values of "new" and "old" represent the digests of the
+            IPTC from the edited and original files respectively, and are undefined if
+            the IPTC does not exist in the respective file.  Set this to "new" as an
+            indication that the XMP is synchronized with the IPTC
         },
         # also note the 'new' feature requires that the IPTC comes before this tag is written
         ValueConv => 'unpack("H*", $val)',
@@ -331,6 +332,7 @@ my %unicodeString = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    DATAMEMBER => [ 1 ],
     FORMAT => 'int16s',
     GROUPS => { 2 => 'Image' },
     0 => {
@@ -341,6 +343,7 @@ my %unicodeString = (
     },
     1 => {
         Name => 'PhotoshopFormat',
+        RawConv => '$$self{PhotoshopFormat} = $val',
         PrintConv => {
             0x0000 => 'Standard',
             0x0001 => 'Optimized',
@@ -349,6 +352,7 @@ my %unicodeString = (
     },
     2 => {
         Name => 'ProgressiveScans',
+        Condition => '$$self{PhotoshopFormat} == 0x0101',
         PrintConv => {
             1 => '3 Scans',
             2 => '4 Scans',
@@ -1143,7 +1147,7 @@ be preserved when copying Photoshop information via user-defined tags.
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
